#!/bin/sh
#
# Copyright 1997, 1998, 1999, 2000  Patrick Volkerding, Moorhead, MN, USA
# Copyright 2001, 2002  Slackware Linux, Inc, Concord, CA, USA
# Copyright 2006, 2009, 2017  Patrick Volkerding, Sebeka, MN, USA
# Copyright 2021 Stephen R Youngs, Brisbane, QLD, Australia
# All rights reserved.
#
# Redistribution and use of this script, with or without modification, is
# permitted provided that the following conditions are met:
#
# 1. Redistributions of this script must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
#  EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
#  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
#  ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

# This is a preprocessor for 'less'.  It is used when this environment
# variable is set:   LESSOPEN="|lesspipe.sh %s"

# 2021-01-23
# A complete re-write eliminating the use of filenames to determine how to
# process a file. --SY
exec 2>/dev/null

lesspipe()
{
    case $(file -Lb --mime-type "$1") in
	application/x-tar) tar tvvf "$1" ;;
        application/x-bzip2)
	    case $(bzip2 -dc "$1"|file -b --mime-type -) in
		application/x-tar) tar tvvf "$1" ;;
		text/troff) bzip2 -dc "$1"|nroff -S -mandoc - ;;
		*) bzip2 -dc "$1" ;;
	    esac ;;
        application/gzip)
	    case $(gzip -dc "$1"|file -b --mime-type -) in
		application/x-tar) tar tvvf "$1" ;;
		text/troff) gzip -dc "$1"|nroff -S -mandoc - ;;
		*) gzip -dc "$1" ;;
	    esac ;;
        application/x-lzip)
	    case $(lzip -dc "$1"|file -b --mime-type -) in
		application/x-tar) tar tvvf "$1" ;;
		text/troff) lzip -dc "$1"|nroff -S -mandoc - ;;
		*) lzip -dc "$1" ;;
	    esac ;;
	application/x-lzma)
	    case $(lzma -dc "$1"|file -b --mime-type -) in
		application/x-tar) tar tvvf "$1" ;;
		text/troff) lzma -dc "$1"|nroff -S -mandoc - ;;
		*) lzma -dc "$1" ;;
	    esac ;;
	application/x-xz)
	    case $(xz -dc "$1"|file -b --mime-type -) in
		application/x-tar) tar tvvf "$1" ;;
		text/troff) xz -dc "$1"|nroff -S -mandoc - ;;
		*) xz -dc "$1" ;;
	    esac ;;
	application/zstd)
	    case $(zstd -dc "$1"|file -b --mime-type -) in
		application/x-tar) tar tvvf "$1" ;;
		text/troff) zstd -dc "$1"|nroff -S -mandoc - ;;
		*) zstd -dc "$1" ;;
	    esac ;;
	application/x-rar) type unrar 1>/dev/null && unrar v "$1" ;;
	application/x-rpm) type rpm 1>/dev/null && rpm -qpvl "$1" ;;
	application/zip) unzip -l "$1" ;;
	application/*debian*) ar -tv "$1" ;;
	text/troff) nroff -S -mandoc "$1" ;;
	text/html) lynx -dump "$1" ;;
	text/xml) head "$1"|grep -q ' html'&& lynx -dump "$1" ;;
        # Lastly, if it is an ELF file, use readelf(1)
        *) file -L "$1"|grep -q ' ELF '&& readelf --all "$1" ;;
    esac
}

lesspipe "$1"

### End lesspipe.sh
